@@ -1,5 +1,5 @@
 /* Hierarchial argument parsing
-   Copyright (C) 1995, 96, 97, 98, 99, 2000 Free Software Foundation, Inc.
+   Copyright (C) 1995, 96, 97, 98, 99, 2000,2003 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Written by Miles Bader <miles@gnu.ai.mit.edu>.
 
@@ -26,12 +26,34 @@
 #include <config.h>
 #endif
 
+/* AIX requires this to be the first thing in the file.  */
+#ifndef __GNUC__
+# if HAVE_ALLOCA_H
+#  include <alloca.h>
+# else
+#  ifdef _AIX
+ #pragma alloca
+#  else
+#   ifndef alloca /* predefined by HP cc +Olibcalls */
+char *alloca ();
+#   endif
+#  endif
+# endif
+#endif
+
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+#if HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 #include <limits.h>
 #include <assert.h>
 
+#if HAVE_MALLOC_H
+/* Needed, for alloca on windows */
+# include <malloc.h>
+#endif
+
 #ifndef _
 /* This is for other GNU distributions with internationalized messages.
    When compiling libc, the _ macro is predefined.  */
@@ -80,7 +102,9 @@ volatile int _argp_hang;
 
 #define OPT_PROGNAME	-2
 #define OPT_USAGE	-3
+#if HAVE_SLEEP && HAVE_GETPID
 #define OPT_HANG	-4
+#endif
 
 static const struct argp_option argp_default_options[] =
 {
@@ -88,8 +112,10 @@ static const struct argp_option argp_default_options[] =
   {"usage",	  OPT_USAGE,	0, 0,  N_("Give a short usage message"), 0 },
   {"program-name",OPT_PROGNAME,"NAME", OPTION_HIDDEN,
      N_("Set the program name"), 0},
+#if OPT_HANG
   {"HANG",	  OPT_HANG,    "SECS", OPTION_ARG_OPTIONAL | OPTION_HIDDEN,
      N_("Hang for SECS seconds (default 3600)"), 0 },
+#endif
   {0, 0, 0, 0, 0, 0}
 };
 
@@ -128,7 +154,8 @@ argp_default_parser (int key, char *arg, struct argp_state *state)
 	state->argv[0] = arg;
 
       break;
-
+      
+#if OPT_HANG
     case OPT_HANG:
       _argp_hang = atoi (arg ? arg : "3600");
       fprintf(state->err_stream, "%s: pid = %ld\n",
@@ -136,7 +163,8 @@ argp_default_parser (int key, char *arg, struct argp_state *state)
       while (_argp_hang-- > 0)
 	__sleep (1);
       break;
-
+#endif
+      
     default:
       return EBADKEY;
     }
