@@ -52,7 +52,7 @@
 #include "parse.h"
 #include "ssh.h"
 #include "suspend.h"
-#include "tcpforward_commands.h"
+#include "tcpforward.h"
 #include "translate_signal.h"
 #include "werror.h"
 #include "xalloc.h"
@@ -67,6 +67,7 @@
 #include "client.c.x"
 
 #define DEFAULT_ESCAPE_CHAR '~'
+#define DEFAULT_SOCKS_PORT 1080
 
 static struct lsh_string *
 format_service_request(int name)
@@ -100,14 +101,8 @@ do_accept_service(struct packet_handler *c,
   
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_SERVICE_ACCEPT)
-      && (
-#if DATAFELLOWS_WORKAROUNDS
-	  (connection->peer_flags & PEER_SERVICE_ACCEPT_KLUDGE)
-#else
-	  0
-#endif
-	  || (parse_atom(&buffer, &name)
-	      && (name == closure->service)))
+      && parse_atom(&buffer, &name)
+      && (name == closure->service)
       && parse_eod(&buffer))
     {
       connection->dispatch[SSH_MSG_SERVICE_ACCEPT] = &connection_fail_handler;
@@ -218,10 +213,9 @@ do_detach_cb(struct lsh_callback *c)
     {
       self->fd_flag = 1; /* Note */
       
-    if (self->channel_flag && self->fd_flag)
-      /* If the fd is closed already, asked to be called from the main loop */ 
-      io_callout(c, 0);
-
+      if (self->channel_flag && self->fd_flag)
+	/* If the fd is closed already, ask to be called from the main loop */ 
+	io_callout(c, 0);
     }
   else
     {
@@ -236,8 +230,7 @@ do_detach_cb(struct lsh_callback *c)
 	  break;
 	  
 	case 0:
-	  /* Detach, lsh doesn't actually use these fds but dup(2)s them */
-	  
+	  /* Detach */	  
 	  close(STDIN_FILENO); 
 	  close(STDOUT_FILENO); 
 	  close(STDERR_FILENO); 
@@ -250,11 +243,6 @@ do_detach_cb(struct lsh_callback *c)
 	  break;
 	  
 	default:
-	  /* It's a good idea to reset std* to blocking mode. */
-	  io_set_blocking(STDIN_FILENO);
-	  io_set_blocking(STDOUT_FILENO);
-	  io_set_blocking(STDERR_FILENO);
-	  
 	  exit(*self->exit_status);
 	}
     }
@@ -526,13 +514,13 @@ make_client_session(struct client_options *options);
 #define OPT_STDOUT 0x211
 #define OPT_STDERR 0x212
 
-#define OPT_FORK_STDIO 0x213
-
 #define OPT_SUBSYSTEM 0x214
 #define OPT_DETACH 0x215
 
 #define OPT_ASKPASS 0x216
 
+#define OPT_WRITE_PID 0x217
+
 void
 init_client_options(struct client_options *self,
 		    struct randomness *random,
@@ -562,15 +550,13 @@ init_client_options(struct client_options *self,
   self->stdout_file = NULL;
   self->stderr_file = NULL;
 
-  self->stdin_fork = 0;
-  self->stdout_fork = 0;
-  self->stderr_fork = 1;
-
   self->used_stdin = 0;
   self->used_pty = 0;
   self->used_x11 = 0;
   
   self->detach_end = 0;
+  self->write_pid = 0;
+  
   self->start_shell = 1;
   self->remote_forward = 0;
 
@@ -590,6 +576,7 @@ client_options[] =
     "Should be an absolute filename.", 0 },
   { NULL, 0, NULL, 0, "Actions:", CLIENT_ARGP_ACTION_GROUP },
   { "forward-local-port", 'L', "local-port:target-host:target-port", 0, "", 0 },
+  { "forward-socks", 'D', "port", OPTION_ARG_OPTIONAL, "Enable socks dynamic forwarding", 0 },
 #if 0
   { "forward-remote-port", 'R', "remote-port:target-host:target-port", 0, "", 0 },
 #endif
@@ -623,13 +610,6 @@ client_options[] =
   { "stderr", OPT_STDERR, "Filename", 0, "Redirect stderr", 0},
   { "no-stderr", OPT_STDERR | ARG_NOT, NULL, 0, "Redirect stderr to /dev/null", 0}, 
 
-  { "cvs-workaround", OPT_FORK_STDIO, "i?o?e?", OPTION_ARG_OPTIONAL,
-    "Avoid setting one or more of the stdio file descriptors into "
-    "non-blocking mode. If no argument is provided, the workaround is "
-    "applied to all three file descriptors. By default, the workaround "
-    "is applied to stderr only.", 0 },
-  { "no-cvs-workaround", OPT_FORK_STDIO | ARG_NOT, NULL, 0,
-    "Disable the cvs workaround.", 0 },
   { "detach", OPT_DETACH, NULL, 0, "Detach from terminal at session end.", 0},
   { "no-detach", OPT_DETACH | ARG_NOT, NULL, 0, "Do not detach session at end," 
     " wait for all open channels (default).", 0},
@@ -638,8 +618,11 @@ client_options[] =
   { "pty", 't', NULL, 0, "Request a remote pty (default).", 0 },
   { "no-pty", 't' | ARG_NOT, NULL, 0, "Don't request a remote pty.", 0 },
 #endif /* WITH_PTY_SUPPORT */
+  { NULL, 0, NULL, 0, "Miscellaneous options:", 0 },
   { "escape-char", 'e', "Character", 0, "Escape char. `none' means disable. "
     "Default is to use `~' if we have a tty, otherwise none.", 0 },
+  { "write-pid", OPT_WRITE_PID, NULL, 0, "Make -B write the pid of the backgrounded "
+    "process to stdout.", 0 },
   { NULL, 0, NULL, 0, NULL, 0 }
 };
 
@@ -826,6 +809,16 @@ client_add_action(struct client_options *options,
   return action;
 }
 
+struct command *
+client_prepend_action(struct client_options *options,
+		      struct command *action)
+{
+  if (action)
+    object_queue_add_head(&options->actions, &action->super);
+
+  return action;
+}
+
 /* NOTE: Some of the original quoting is lost here. */
 static struct lsh_string *
 rebuild_command_line(unsigned argc, char **argv)
@@ -860,60 +853,6 @@ rebuild_command_line(unsigned argc, char **argv)
   return r;
 }
 
-static int
-fork_input(int in)
-{
-  /* pipe[0] for reading, pipe[1] for writing. */
-  int pipe[2];
-
-  if (!lsh_make_pipe(pipe))
-    return -1;
-
-  switch (fork())
-    {
-    case -1:
-      /* Error */
-      return -1;
-    case 0:
-      close(pipe[0]);
-      if (lsh_copy_file(in, pipe[1]))
-	_exit(EXIT_SUCCESS);
-      else
-	_exit(EXIT_FAILURE);
-    default:
-      /* Parent */
-      close(pipe[1]);
-      return pipe[0];
-    }
-}
-
-static int
-fork_output(int out)
-{
-  /* pipe[0] for reading, pipe[1] for writing. */
-  int pipe[2];
-
-  if (!lsh_make_pipe(pipe))
-    return -1;
-
-  switch (fork())
-    {
-    case -1:
-      /* Error */
-      return -1;
-    case 0:
-      close(pipe[1]);
-      if (lsh_copy_file(pipe[0], out))
-	_exit(EXIT_SUCCESS);
-      else
-	_exit(EXIT_FAILURE);
-    default:
-      /* Parent */
-      close(pipe[0]);
-      return pipe[1];
-    }
-}
-
 /* A callback that exits the process immediately. */
 DEFINE_ESCAPE(exit_callback, "Exit.")
 {
@@ -941,13 +880,27 @@ DEFINE_ESCAPE(quiet_callback, "Toggle warning messages.")
     werror("Enabling warning messages\n");
 }
 
-DEFINE_COMMAND(background_process)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
+/* GABA:
+   (class
+     (name background_process_command)
+     (super command)
+     (vars
+       (write_pid . int)))
+*/
+
+static void
+do_background_process(struct command *s,
+		      struct lsh_object *a,
+		      struct command_continuation *c,
+		      struct exception_handler *e UNUSED)
 {
-  switch (fork())
+  CAST(background_process_command, self, s);
+
+  trace("do_background_process\n");
+  
+  pid_t pid = fork();
+  
+  switch (pid)
     {
     case 0:
       /* Child */
@@ -962,10 +915,29 @@ DEFINE_COMMAND(background_process)
       break;
     default:
       /* Parent */
+      if (self->write_pid)
+	{
+	  struct lsh_string *msg = ssh_format("%di\n", pid);
+	  const struct exception *e = write_raw (STDOUT_FILENO, STRING_LD(msg));
+
+	  if (e)
+	    werror ("Write to stdout failed!?: %z\n", e->msg);
+	}
       _exit(EXIT_SUCCESS);
     }
 }
 
+static struct command *
+make_background_process(int write_pid)
+{
+  NEW(background_process_command, self);
+
+  self->super.call = do_background_process;
+  self->write_pid = write_pid;
+
+  return &self->super;
+}
+
 /* Create a session object. stdout and stderr are shared (although
  * with independent lsh_fd objects). stdin can be used by only one
  * session (until something "session-control"/"job-control" is added).
@@ -976,6 +948,10 @@ make_client_session(struct client_options *options)
   int in;
   int out;
   int err;
+  enum io_type in_type = IO_NORMAL;
+  enum io_type out_type = IO_NORMAL;
+  enum io_type err_type = IO_NORMAL;
+  
   int is_tty = 0;
   struct ssh_channel *session;
   
@@ -986,13 +962,15 @@ make_client_session(struct client_options *options)
 
   if (options->stdin_file)
     in = open(options->stdin_file, O_RDONLY);
+      
   else
     {
       if (options->used_stdin)
 	in = open("/dev/null", O_RDONLY);
       else 
 	{
-	  in = (options->stdin_fork ? fork_input : dup)(STDIN_FILENO);
+	  in = STDIN_FILENO;
+	  in_type = IO_STDIO;
 	  is_tty = isatty(STDIN_FILENO);
 	  
 	  options->used_stdin = 1;
@@ -1001,7 +979,7 @@ make_client_session(struct client_options *options)
 
   if (in < 0)
     {
-      werror("lsh: Can't dup/open stdin %e\n", errno);
+      werror("lsh: Can't open stdin %e\n", errno);
       return NULL;
     }
 
@@ -1036,14 +1014,14 @@ make_client_session(struct client_options *options)
 
   if (options->stdout_file)
     out = open(options->stdout_file, O_WRONLY | O_CREAT, 0666);
-  else if (options->stdout_fork)
-    out = fork_output(STDOUT_FILENO);
   else
-    out = dup(STDOUT_FILENO);
-
+    {
+      out_type = IO_STDIO;
+      out = STDOUT_FILENO;
+    }
   if (out < 0)
     {
-      werror("lsh: Can't dup/open stdout %e\n", errno);
+      werror("lsh: Can't open stdout %e\n", errno);
       close(in);
       return NULL;
     }
@@ -1052,17 +1030,14 @@ make_client_session(struct client_options *options)
   
   if (options->stderr_file)
     err = open(options->stderr_file, O_WRONLY | O_CREAT, 0666);
-  else if (options->stderr_fork)
-    err = fork_output(STDERR_FILENO);
   else
     {
-      err = dup(STDERR_FILENO);
-      set_error_nonblocking(STDERR_FILENO);
+      err_type = IO_STDIO;
+      err = STDERR_FILENO;
     }
-
   if (err < 0) 
     {
-      werror("lsh: Can't dup/open stderr!\n");
+      werror("lsh: Can't open stderr!\n");
       close(in);
       close(out);
       return NULL;
@@ -1075,13 +1050,12 @@ make_client_session(struct client_options *options)
   options->stdin_file = options->stdout_file = options->stderr_file = NULL;
 
   session = make_client_session_channel
-    (io_read(make_lsh_fd(in, "client stdin", options->handler),
+    (io_read(make_lsh_fd(in, in_type, "client stdin", options->handler),
 	     NULL, NULL),
-     io_write(make_lsh_fd(out, "client stdout", options->handler),
+     io_write(make_lsh_fd(out, out_type, "client stdout", options->handler),
 	      BLOCK_SIZE, 
-	      detach_cb
-	      ),
-     io_write(make_lsh_fd(err, "client stderr", options->handler),
+	      detach_cb),
+     io_write(make_lsh_fd(err, err_type, "client stderr", options->handler),
 	      BLOCK_SIZE, NULL),
      escape,
      WINDOW_SIZE,
@@ -1195,6 +1169,17 @@ client_parse_forward_arg(char *arg,
   return 1;
 }
 
+static int
+client_arg_unsigned(const char *arg, unsigned long *n)
+{
+  char *end;
+  if (*arg == 0)
+    return 0;
+
+  *n = strtoul(arg, &end, 0);
+  return *end == 0;
+}
+		    
 #define CASE_ARG(opt, attr, none)		\
   case opt:					\
     if (options->not)				\
@@ -1333,7 +1318,7 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 	struct address_info *target;
 
 	if (!client_parse_forward_arg(arg, &listen_port, &target))
-	  argp_error(state, "Invalid forward specification '%s'.", arg);
+	  argp_error(state, "Invalid forward specification `%s'.", arg);
 
 	client_add_action(options, make_forward_local_port
 			  (make_address_info((options->with_remote_peers
@@ -1344,13 +1329,27 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 	break;
       }      
 
+    case 'D':
+      {
+	unsigned long socks_port = DEFAULT_SOCKS_PORT;
+	if (arg && (client_arg_unsigned(arg, &socks_port) == 0 || socks_port > 0xffff))
+	  argp_error(state, "Invalid port number `%s' for socks.", arg);
+
+	client_add_action(options, make_socks_server
+			  (make_address_info((options->with_remote_peers
+					      ? NULL
+					      : ssh_format("%lz", "127.0.0.1")),
+					     socks_port)));
+	break;
+      }
+      
     case 'N':
       options->start_shell = 0;
       break;
 
     case 'B':
       options->start_shell = 0;
-      client_add_action(options, &background_process);
+      client_add_action(options, make_background_process(options->write_pid));
       break;
       
     CASE_FLAG('g', with_remote_peers);
@@ -1360,50 +1359,12 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
 #endif /* WITH_PTY_SUPPORT */
 
     CASE_FLAG(OPT_DETACH, detach_end);
-
+    CASE_FLAG(OPT_WRITE_PID, write_pid);
+    
     CASE_ARG(OPT_STDIN, stdin_file, "/dev/null");
     CASE_ARG(OPT_STDOUT, stdout_file, "/dev/null"); 
     CASE_ARG(OPT_STDERR, stderr_file, "/dev/null");
 
-    case OPT_FORK_STDIO:
-      if (options->not)
-	{
-	  options->not = 0;
-	case OPT_FORK_STDIO | ARG_NOT:
-	  options->stdin_fork = options->stdout_fork = options->stderr_fork = 0;
-	  break;
-	}
-      else if (!arg)
-	options->stdin_fork = options->stdout_fork = options->stderr_fork = 1;
-      else
-	{
-	  int i;
-
-	  options->stdin_fork = options->stdout_fork = options->stderr_fork = 0;
-
-	  for (i = 0; arg[i]; i++)
-	    switch(arg[i])
-	      {
-	      case 'i': case 'I':
-		options->stdin_fork = 1;
-		break;
-	      case 'o': case 'O':
-		options->stdout_fork = 1;
-		break;
-	      case 'e': case 'E':
-		options->stderr_fork = 1;
-		break;
-	      default:
-		argp_error(state, "The argument to --cvs-workaround should "
-			   "be one or more of the characters 'i' (stdin), "
-			   "'o' (stdout) and 'e' (stderr).");
-		goto loop_done;
-	      }
-	loop_done:
-          ;
-	}
-      break;
-		
     case 'n':
       options->not = !options->not;
       break;
