@@ -160,16 +160,13 @@ make_client_x11_display(const char *display, struct lsh_string *fake);
        (stdout_file . "const char *")
        (stderr_file . "const char *")
 
-       ; fork() extra processes for handling stdio file-descriptors,
-       ; to avoid setting them in non-blocking mode.
-       (stdin_fork . int)
-       (stdout_fork . int)
-       (stderr_fork . int)
-
        ; True if the process's stdin or pty (respectively) has been used. 
        (used_stdin . int)
        (used_pty . int)
        (used_x11 . int)
+
+       ; Should -B write the pid to stdout?
+       (write_pid . int)
        
        ; True if the client should detach when a session closes (useful for gateways)
        (detach_end . int)
@@ -191,6 +188,10 @@ init_client_options(struct client_options *options,
 struct command *
 client_add_action(struct client_options *options,
 		  struct command *action);
+struct command *
+client_prepend_action(struct client_options *options,
+		      struct command *action);
+
 int
 client_parse_forward_arg(char *arg,
 			 uint32_t *listen_port,
