@@ -26,7 +26,6 @@
 
 #include "alist.h"
 #include "connection.h"
-#include "ssh1_fallback.h"
 
 #define GABA_DECLARE
 #include "handshake.h.x"
@@ -47,9 +46,7 @@
        (random object randomness)
        (algorithms object alist)
 
-       (banner_text string)
-       ; Used only on the server
-       (fallback object ssh1_fallback)))
+       (banner_text string)))
 */
 
 struct handshake_info *
@@ -59,7 +56,6 @@ make_handshake_info(enum connection_flag flags,
 		    uint32_t block_size,
 		    struct randomness *r,
 		    struct alist *algorithms,
-		    struct ssh1_fallback *fallback,
 		    struct lsh_string *banner_text);
 
 extern struct command_4 handshake_command;
