@@ -76,7 +76,7 @@ make_keypair(uint32_t type,
        (add method int "mpz_t res" "mpz_t a" "mpz_t b")
        (subtract method int "mpz_t res" "mpz_t a" "mpz_t b")
        
-       ; FIXME: Doesn't handle negative exponents
+       ; NOTE: Doesn't handle negative exponents
        (power method void "mpz_t res" "mpz_t g" "mpz_t e")
        (small_power method void "mpz_t res" "mpz_t g" "uint32_t e"))) */
 
@@ -209,11 +209,7 @@ struct verifier *
 parse_ssh_rsa_public(struct simple_buffer *buffer);
 
 struct verifier *
-make_ssh_rsa_verifier(uint32_t public_length,
-		      const uint8_t *public);
-
-struct lsh_string *
-rsa_generate_key(struct randomness *r, uint32_t bits);
+make_ssh_rsa_verifier(const struct lsh_string *public);
 
 
 /* DSA signatures */
@@ -226,13 +222,7 @@ struct verifier *
 parse_ssh_dss_public(struct simple_buffer *buffer);
 
 struct verifier *
-make_ssh_dss_verifier(uint32_t public_length,
-		      const uint8_t *public);
+make_ssh_dss_verifier(const struct lsh_string *public);
 
-void dsa_nist_gen(mpz_t p, mpz_t q, struct randomness *r, unsigned l);
-void dsa_find_generator(mpz_t g, struct randomness *r, mpz_t p, mpz_t q);
-
-struct lsh_string *
-dsa_generate_key(struct randomness *r, unsigned level);
 
 #endif /* LSH_PUBLICKEY_CRYPTO_H_INCLUDED */
