@@ -30,6 +30,11 @@
 #include <assert.h>
 #include <string.h>
 
+#include "nettle/sexp.h"
+
+#include "spki/parse.h"
+#include "spki/tag.h"
+
 #include "spki.h"
 
 #include "atoms.h"
@@ -38,6 +43,7 @@
 #include "io.h"
 #include "interact.h"
 #include "list.h"
+#include "lsh_string.h"
 #include "parse.h"
 #include "publickey_crypto.h"
 #include "randomness.h"
@@ -46,11 +52,6 @@
 #include "xalloc.h"
 #include "alist.h"
 
-#include "nettle/sexp.h"
-
-#include "spki/parse.h"
-#include "spki/tag.h"
-
 
 /* Forward declarations */
 static void
@@ -69,10 +70,10 @@ do_spki_acl_db_free(struct spki_acl_db *db);
  * dns fqdn:s. */
 
 struct lsh_string *
-make_ssh_hostkey_tag(struct address_info *host)
+make_ssh_hostkey_tag(const char *host)
 {
-  uint32_t left = host->ip->length;
-  uint8_t *s = host->ip->data;
+  uint32_t left = strlen(host);
+  const uint8_t *s = host;
   struct lsh_string *tag;
   struct lsh_string *reversed = lsh_string_alloc(left);
 
@@ -82,7 +83,7 @@ make_ssh_hostkey_tag(struct address_info *host)
       uint8_t *p = memchr(s, '.', left);
       if (!p)
 	{
-	  memcpy(reversed->data, s, left);
+	  lsh_string_write(reversed, 0, left, s);
 	  break;
 	}
       else
@@ -90,14 +91,14 @@ make_ssh_hostkey_tag(struct address_info *host)
 	  uint32_t segment = p - s;
 	  left -= segment;
 
-	  memcpy(reversed->data + left, s, segment);
-	  reversed->data[--left] = '.';
+	  lsh_string_write(reversed, left, segment, s);
+	  lsh_string_putc(reversed, --left, '.');
 	  s = p+1;
 	}
     }
 
-  tag = lsh_sexp_format(0, "(tag(ssh-hostkey%s))",
-			reversed->length, reversed->data);
+  tag = lsh_string_format_sexp(0, "(tag(ssh-hostkey%s))",
+			       STRING_LD(reversed));
   lsh_string_free(reversed);
 
   return tag;
@@ -178,17 +179,17 @@ spki_make_signer(struct alist *algorithms,
 		 int *algorithm_name)
 {
   struct sexp_iterator i;
-  uint8_t *decoded;  
-
-  decoded = alloca(key->length);
-  memcpy(decoded, key->data, key->length);
+  struct lsh_string *decoded = lsh_string_dup(key);
+  struct signer *res = NULL;
   
-  if (sexp_transport_iterator_first(&i, key->length, decoded)
+  if (lsh_string_transport_iterator_first(decoded, &i)
       && sexp_iterator_check_type(&i, "private-key"))
-    return spki_sexp_to_signer(algorithms, &i, algorithm_name);
+    res = spki_sexp_to_signer(algorithms, &i, algorithm_name);
 
-  werror("spki_make_signer: Expected private-key expression.\n");
-  return NULL;
+  lsh_string_free(decoded);
+  if (!res)
+    werror("spki_make_signer: Expected private-key expression.\n");
+  return res;
 }
 
 struct lsh_string *
@@ -197,14 +198,19 @@ spki_hash_data(const struct hash_algorithm *algorithm,
 	       uint32_t length, uint8_t *data)
 {
   struct hash_instance *hash = make_hash(algorithm);
-  uint8_t *out = alloca(HASH_SIZE(hash));
-
+  struct lsh_string *digest;
+  struct lsh_string *out;
+  
   hash_update(hash, length, data);
-  hash_digest(hash, out);
+  digest = hash_digest_string(hash);
 
-  return lsh_sexp_format(0, "(hash%0s%s)",
-			 "hash", get_atom_name(algorithm_name),
-			 HASH_SIZE(hash), out);
+  out = lsh_string_format_sexp(0, "(hash%0s%s)",
+			       "hash", get_atom_name(algorithm_name),
+			       STRING_LD(digest));
+  KILL(hash);
+  lsh_string_free(digest);
+
+  return out;
 }  
 
 static void
@@ -275,7 +281,7 @@ spki_authorize(struct spki_context *self,
   struct spki_iterator i;
   const struct spki_5_tuple_list *p;
   
-  if (!spki_iterator_first(&i, access->length, access->data)
+  if (!spki_iterator_first(&i, STRING_LD(access))
       || !spki_parse_tag(&self->db, &i, &tag))
     {
       werror("spki_authorize: Invalid tag.\n");
@@ -337,7 +343,7 @@ spki_pkcs5_encrypt(struct randomness *r,
 		   uint32_t salt_length,
 		   struct lsh_string *password,
 		   uint32_t iterations,
-                   struct lsh_string *data)
+                   const struct lsh_string *data)
 {
   struct lsh_string *key;
   struct lsh_string *salt;
@@ -349,41 +355,33 @@ spki_pkcs5_encrypt(struct randomness *r,
   assert(prf);
 
   /* NOTE: Allows random to be of bad quality */
-  salt = lsh_string_alloc(salt_length);
-  RANDOM(r, salt->length, salt->data);
-    
-  key = lsh_string_alloc(crypto->key_size);
+  salt = lsh_string_random(r, salt_length);
 
-  pkcs5_derive_key(prf,
-		   password->length, password->data,
-		   salt->length, salt->data,
-		   iterations,
-		   key->length, key->data);
+  key = pkcs5_derive_key(prf,
+			 password, salt, iterations,
+			 crypto->key_size);
 
   if (crypto->iv_size)
-    {
-      iv = lsh_string_alloc(crypto->iv_size);
-      RANDOM(r, iv->length, iv->data);
-    }
+    iv = lsh_string_random(r, crypto->iv_size);
   else
     iv = NULL;
 
   encrypted = crypt_string_pad(MAKE_ENCRYPT(crypto,
-					    key->data,
-					    iv ? iv->data : NULL),
+					    lsh_string_data(key),
+					    iv ? lsh_string_data(iv) : NULL),
 			       data, 0);
   
   /* FIXME: Handle iv == NULL. */
-  value = lsh_sexp_format(0, "(password-encrypted%s(xpkcs5v2%0s"
-			  "(iterations%i)(salt%s))"
-			  "(%0s(iv%s)(data%s)))",
-			  label->length, label->data,
-			  get_atom_name(prf_name),
-			  iterations,
-			  salt->length, salt->data,
-			  get_atom_name(crypto_name),
-			  iv->length, iv->data,
-			  encrypted->length, encrypted->data);
+  value = lsh_string_format_sexp(0, "(password-encrypted%s(Xpkcs5v2%0s"
+				 "(iterations%i)(salt%s))"
+				 "(%0s(iv%s)(data%s)))",
+				 STRING_LD(label),
+				 get_atom_name(prf_name),
+				 iterations,
+				 STRING_LD(salt),
+				 get_atom_name(crypto_name),
+				 STRING_LD(iv),
+				 STRING_LD(encrypted));
 
   lsh_string_free(key);
   lsh_string_free(salt);
@@ -393,6 +391,85 @@ spki_pkcs5_encrypt(struct randomness *r,
   return value;
 }
 
+static int
+parse_pkcs5(struct sexp_iterator *i, struct alist *mac_algorithms,
+	    struct mac_algorithm **mac, uint32_t *iterations,
+	    const struct lsh_string **salt)
+{
+  switch (lsh_sexp_get_type(i)) 
+    {
+    default:
+      werror("Unknown key derivation mechanism.\n");
+      return 0;
+
+    case ATOM_XPKCS5V2:
+      {
+	const uint8_t *names[2] = { "salt", "iterations" };
+	struct sexp_iterator values[2];
+	    
+	CAST_SUBTYPE(mac_algorithm, tmp,
+		     ALIST_GET(mac_algorithms, lsh_sexp_to_atom(i)));
+
+	*mac = tmp;
+	if (!*mac)
+	  {
+	    werror("Unknown mac for pkcs5v2.\n");
+	    return 0;
+	  }
+
+	return (sexp_iterator_assoc(i, 2, names, values)
+		&& (*salt = lsh_sexp_to_string(&values[0], NULL))
+		&& sexp_iterator_get_uint32(&values[1], iterations)
+		&& *iterations);
+      }
+    }
+}
+
+static int
+parse_pkcs5_payload(struct sexp_iterator *i, struct alist *crypto_algorithms,
+		    struct crypto_algorithm **crypto,
+		    const struct lsh_string **iv,
+		    const struct lsh_string **data)
+{
+  const uint8_t *names[2] = { "data", "iv" };
+  struct sexp_iterator values[2];
+
+  CAST_SUBTYPE(crypto_algorithm, tmp,
+	       spki_algorithm_lookup(crypto_algorithms, i, NULL));
+	
+  *crypto = tmp;
+
+  if (!*crypto)
+    {
+      werror("Unknown encryption algorithm for pkcs5v2.\n");
+      return 0;
+    }
+
+  if ((*crypto)->iv_size)
+    {
+      if (!sexp_iterator_assoc(i, 2, names, values))
+	return 0;
+
+      *iv = lsh_sexp_to_string(&values[1], NULL);
+
+      if (lsh_string_length(*iv) != (*crypto)->iv_size)
+	return 0;
+    }
+  else if (!sexp_iterator_assoc(i, 1, names, values))
+    return 0;
+
+  *data = lsh_sexp_to_string(&values[0], NULL);
+    
+  if ((*crypto)->block_size
+      && (lsh_string_length(*data) % (*crypto)->block_size))
+    {
+      werror("Payload data doesn't match block size for pkcs5v2.\n");
+      return 0;
+    }
+
+  return 1;
+}
+
 /* Frees input string. */
 struct lsh_string *
 spki_pkcs5_decrypt(struct alist *mac_algorithms,
@@ -402,19 +479,16 @@ spki_pkcs5_decrypt(struct alist *mac_algorithms,
 {
   struct sexp_iterator i;
   
-  if (! (sexp_iterator_first(&i, expr->length, expr->data)
-	 && sexp_iterator_check_type(&i, "password_encrypted")))
+  if (! (sexp_iterator_first(&i, STRING_LD(expr))
+	 && sexp_iterator_check_type(&i, "password-encrypted")))
     return expr;
 
   else
     {
-      const struct lsh_string *label;
-      struct sexp_iterator key_info;
-
       struct crypto_algorithm *crypto;
       struct mac_algorithm *mac;
 
-      /* FIXME: Leaks some strings. */
+      const struct lsh_string *label = NULL;
       const struct lsh_string *salt = NULL;
       const struct lsh_string *iv = NULL;
       const struct lsh_string *data = NULL;
@@ -432,139 +506,54 @@ spki_pkcs5_decrypt(struct alist *mac_algorithms,
 	  lsh_string_free(expr);
 	  lsh_string_free(iv);
 	  lsh_string_free(salt);
+	  lsh_string_free(label);
 	  return NULL;
 	}
 
-      key_info = i;
-
-      /* Examine the payload expression first, before asking for a
-       * pass phrase. */
-
-      if (!sexp_iterator_next(&i))
+      if (!parse_pkcs5(&i, mac_algorithms, &mac, &iterations, &salt))
 	goto fail;
 
-      if (sexp_iterator_enter_list(&i))
+      if (!parse_pkcs5_payload(&i, crypto_algorithms,
+			       &crypto, &iv, &data))
 	goto fail;
       
+      /* Do the work */
+      
       {
-	const uint8_t *names[2] = { "data", "iv" };
-	struct sexp_iterator values[2];
-
-	CAST_SUBTYPE(crypto_algorithm, tmp,
-		     spki_algorithm_lookup(crypto_algorithms, &i, NULL));
+	struct lsh_string *password
+	  = INTERACT_READ_PASSWORD(interact, 500,
+				   ssh_format("Passphrase for key `%lS': ",
+					      label));
+	struct lsh_string *clear;
+	struct lsh_string *key;
 	
-	crypto = tmp;
-
-	if (!crypto)
+	if (!password)
 	  {
-	    werror("Unknown encryption algorithm for pkcs5v2.\n");
+	    werror("No password provided for pkcs5v2.\n");
 	    goto fail;
 	  }
 
-	if (!sexp_iterator_assoc(&i, crypto->iv_size ? 2 : 1,
-				 names, values))
-	  goto fail;
+	key = pkcs5_derive_key(mac,
+			       password, salt, iterations,
+			       crypto->key_size);
 
-	data = lsh_sexp_to_string(&values[0], NULL);
-	iv = crypto->iv_size ? lsh_sexp_to_string(&values[1], NULL) : NULL;
-      }
-	
-      if (crypto->iv_size)
-	{
-	  if (!iv || (iv->length != crypto->iv_size))
-	    {
-	      werror("Invalid IV for pkcs5v2.\n");
-	      goto fail;
-	    }
-	}
-      else if (iv)
-	{
-	  if (iv->length)
-	    {
-	      werror("Unexpected iv provided for pkcs5v2.\n");
-	      goto fail;
-	    }
-	  lsh_string_free(iv);
-	  iv = NULL;
-	}
-	
-      if (crypto->block_size && (data->length % crypto->block_size))
-	{
-	  werror("Payload data doesn't match block size for pkcs5v2.\n");
-	  goto fail;
-	}
-
-      /* Get key */
-      i = key_info;
-      switch (lsh_sexp_get_type(&i)) 
-	{
-	default:
-	  werror("Unknown key derivation mechanism.\n");
-	  goto fail;
-
-	case ATOM_XPKCS5V2:
-	  {
-	    const uint8_t *names[2] = { "salt", "iterations" };
-	    struct sexp_iterator values[2];
-	    
-	    CAST_SUBTYPE(mac_algorithm, tmp,
-			 spki_algorithm_lookup(mac_algorithms,
-					       &i, NULL));
-
-	    mac = tmp;
-
-	    if (! (sexp_iterator_assoc(&i, 2, names, values)
-		   && (salt = lsh_sexp_to_string(&values[0], NULL))
-		   && sexp_iterator_get_uint32(&values[1], &iterations)
-		   && iterations))
-	      goto fail;
-	  }
-
-	  if (!mac)
-	    {
-	      werror("Unknown mac for pkcs5v2.\n");
-	      goto fail;
-	    }
-
-	  /* Do the work */
-
-	  {
-	    struct lsh_string *password
-	      = INTERACT_READ_PASSWORD(interact, 500,
-				       ssh_format("Passphrase for key `%lS': ",
-						  label), 1);
-	    struct lsh_string *clear;
-	    uint8_t *key;
-	    
-	    if (!password)
-	      {
-		werror("No password provided for pkcs5v2.");
-		goto fail;
-	      }
-
-	    key = alloca(crypto->key_size);
-	    pkcs5_derive_key(mac,
-			     password->length, password->data,
-			     salt->length, salt->data,
-			     iterations,
-			     crypto->key_size, key);
-	    
-	    clear = crypt_string_unpad(MAKE_DECRYPT(crypto,
-						    key,
-						    iv ? iv->data : NULL),
-				       data, 0);
-
-	    lsh_string_free(data);
-	    lsh_string_free(expr);
-	    lsh_string_free(iv);
-	    lsh_string_free(password);
-	    lsh_string_free(salt);
+	clear
+	  = crypt_string_unpad(MAKE_DECRYPT(crypto,
+					    lsh_string_data(key),
+					    iv ? lsh_string_data(iv) : NULL),
+			       data, 0);
+	lsh_string_free(data);
+	lsh_string_free(expr);
+	lsh_string_free(iv);
+	lsh_string_free(password);
+	lsh_string_free(salt);
+	lsh_string_free(label);
+	lsh_string_free(key);
 	    	    
-	    if (!clear)
-	      werror("Bad password for pkcs5v2.\n");
+	if (!clear)
+	  werror("Bad password for pkcs5v2.\n");
 
-	    return clear;
-	  }
-	}
+	return clear;
+      }
     }
 }
