@@ -181,13 +181,6 @@ set_error_stream(int fd)
   error_write = write_raw;
 }
 
-void
-set_error_nonblocking(int fd)
-{
-  if (error_fd == fd)
-    error_write = write_raw_with_poll;
-}
-
 void
 set_error_raw(int raw)
 {
